@@ -17,11 +17,6 @@ ENV['NO_PROXY'] ||= ENV['UNICORN_TEST_ADDR'] || '127.0.0.1'
 DEFAULT_TRIES = 1000
 DEFAULT_RES = 0.2
 
-HERE = File.dirname(__FILE__) unless defined?(HERE)
-%w(lib ext).each do |dir|
-  $LOAD_PATH.unshift "#{HERE}/../#{dir}"
-end
-
 require 'test/unit'
 require 'net/http'
 require 'digest/sha1'
@@ -77,6 +72,7 @@ def hit(uris)
     res = nil
 
     if u.kind_of? String
+      u = 'http://127.0.0.1:8080/' if u == 'http://0.0.0.0:8080/'
       res = Net::HTTP.get(URI.parse(u))
     else
       url = URI.parse(u[0])
